"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _httpErrors = _interopRequireDefault(require("http-errors"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Custom Verdaccio Authenticate Plugin.
 */
class AclPlugin {
  constructor(config, options) {
    _defineProperty(this, "logger", void 0);

    _defineProperty(this, "userGroups", void 0);

    this.logger = options.logger;
    this.userGroups = new Array();
    Object.keys(config).forEach(groupName => {
      var users = this.splitString(config[groupName], ' ');
      users.forEach(userName => {
        if (!this.userGroups[userName]) {
          this.userGroups[userName] = [];
        }

        this.userGroups[userName].push(groupName);
      });
    });
    return this;
  }

  allowAction(action, user, pkg, cb) {
    const userName = user.name;
    const allowedGroups = pkg[action];
    const currentUserGroups = new Array();
    user.groups.forEach(_rg => {
      currentUserGroups.push(_rg);
    }); // @ts-ignore

    if (this.userGroups[userName]) {
      // @ts-ignore
      this.userGroups[userName].forEach(_rg => {
        currentUserGroups.push(_rg);
      });
    }

    let allowed = false;
    currentUserGroups.forEach(_group => {
      if (allowedGroups.indexOf(_group) > -1) {
        console.log(userName + 'is allowed to perform "' + action + '" because he belongs to group ' + _group);
        allowed = true;
      }
    });

    if (allowed) {
      return cb(null, true);
    }

    if (userName) {
      cb((0, _httpErrors.default)(403, `'${action}' not allowed on package '${pkg}' for user '${userName}'`), false);
    } else {
      cb((0, _httpErrors.default)(401, `Authentication is required to perform ${action} in package ${pkg}`), false);
    }
  }
  /**
   * Authenticate an user.
   * @param user user to log
   * @param password provided password
   * @param cb callback function
   */


  authenticate(user, password, cb) {
    cb(null, [user]);
  }
  /**
   * Triggered on each access request
   * @param user
   * @param pkg
   * @param cb
   */


  allow_access(user, pkg, cb) {
    return this.allowAction(AclPlugin.ACCESS_ACTION, user, pkg, cb);
  }
  /**
   * Triggered on each publish request
   * @param user
   * @param pkg
   * @param cb
   */


  allow_publish(user, pkg, cb) {
    return this.allowAction(AclPlugin.PUBLISH_ACTION, user, pkg, cb);
  }

  allow_unpublish(user, pkg, cb) {
    return this.allowAction(AclPlugin.UNPUBLISH_ACTION, user, pkg, cb);
  }

  splitString(str, separator) {
    var index = 0;
    var token = "";
    var newStr = str + " ";
    var split = new Array();

    for (var j = 0; j < newStr.length; j++) {
      if (newStr[j] === separator) {
        split.push(token);
        index++;
        token = "";
      } else {
        token += newStr[j];
      }
    }

    return split;
  }

}

exports.default = AclPlugin;

_defineProperty(AclPlugin, "ACCESS_ACTION", 'access');

_defineProperty(AclPlugin, "PUBLISH_ACTION", 'publish');

_defineProperty(AclPlugin, "UNPUBLISH_ACTION", 'unpublish');